#include "lcd.h"

/* Variable information */
static struct fb_var_screeninfo vinfo;
/* Fixed information */
static struct fb_fix_screeninfo finfo;
/* Screen size */
static int screen_size = 0;
/* Point to map framebuffer of device */
static short *fbp = NULL;
/* Point to old map framebuffer,
   used to restitute LCD framebuffer */
static short *old_fbp = NULL;
/* File description of framebuffer */
static int fd_fb = 0;
/* The status of initialize bramebuffer */
static RET_STATUS init_flag = FAILD;

/*************************************************************//*
 * Private function
 * Backup framebuffer
 *************************************************************/
static void backup_fb(void);

/*************************************************************//*
 * Private function
 * Restitute framebuffer
 *************************************************************/
static void restitute_fb(void);

/*************************************************************//*
 * Initialize lcd
 * parameter
 *         dev_pathname: The pathname of LCD device framebuffer
 *
 * return
 *         Return initialize status, SUCCESS represent initialize
 *         success and FAILD represent faild
 ************************************************************/
RET_STATUS init_lcd(const char *dev_pathname)
{
	/* Open framebuffer and get that's file description */
	fd_fb = open(dev_pathname, O_RDWR);
	if (!fd_fb) {
		perror("Error open framebuffer device: ");
		return FAILD;
	}

	/* Get fixed screen information */
	if (ioctl(fd_fb, FBIOGET_FSCREENINFO, &finfo)) {
		perror("Error reading fixed information: ");
		return FAILD;
	}

	/* Get variable screen information */
	if (ioctl(fd_fb, FBIOGET_VSCREENINFO, &vinfo)) {
		perror("Error reading variable information: ");
		return FAILD;
	}

	/* The size of screen in bytes */
	screen_size = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;

	/* Map framebuffer to memory */
	fbp = (short*)mmap(0, screen_size, PROT_READ | PROT_WRITE,
						  MAP_SHARED, fd_fb, 0);
	if ((int)fbp == -1) {
		printf("Error: failed to map framebuffer device to memory.\n");
		return FAILD;
	}

	printf("Initialize LCD and framebuffer successful\n");
	
	init_flag = SUCCESS;
	backup_fb(); // Backup framebuffer
	
	return init_flag;
}

/*************************************************************//*
 * Print information of LCD that has been initialize
 * parameter
 *         None
 *
 * return
 *         None
 ************************************************************/
void print_lcdinfo(void)
{
	if (!init_flag) {
		printf("LCD hasn't initialized\n");
		return;
	}
	
	printf("%d x %d, %dbpp, screen_size = %d\n",
		   vinfo.xres, vinfo.yres, vinfo.bits_per_pixel, screen_size);
}

/*************************************************************//*
 * Draw on LCD
 * parameter
 *         start: The start position on LCD
 *         end:   The end position on LCD
 *         color: The value of color
 *
 * return
 *         None
 ************************************************************/
void draw_region(const long int start,
				 const long int end,
				 const short color)
{
	long int inc;

	if (!init_flag) {
		printf("LCD hasn't initialized\n");
		return;
	}
	
	if (end < start) {
		return;
	}

	/* Set points color value that on the specify region */ 
	for(inc = start; inc < end; ++inc) {
		*(fbp + inc) = color;
	}
}

/*************************************************************//*
 * Draw a rectangle on LCD
 * parameter
 *         start_x: The start point x position on LCD
 *         start_y: The start point y position on LCD
 *         end_x:   The end point x position on LCD
 *         end_y:   The end point y position on LCD
 *
 * return
 *         None
 ************************************************************/
void draw_rectangle(long int start_x,
					long int start_y,
					const long int end_x,
					const long int end_y,
					const short color)
{
	long int start_point;
	long int x;
	long int y;
	
	if (end_x < 0 ||
		end_y < 0 ||
		start_x > end_x ||
		start_y > end_y) {
		return;
	}
	
	if (start_x < 0) {
		start_x = 0;
	}
	if (start_y < 0) {
		start_y = 0;
	}

	for (y = start_y; y < end_y; ++y) {
		start_point = y * vinfo.xres; 
		for (x = start_x; x < end_x; ++x) {
			*(fbp + start_point + x) = color;
		}
	}
}

/*************************************************************//*
 * Restitute and close the file description
 * parameter
 *         None
 *
 * return
 *         None
 ************************************************************/
void restitute_lcd(void)
{
	restitute_fb();

	/* Unmap framebuffer */
	munmap(fbp, screen_size);
	/* Delete memory */
	free(old_fbp);
	
	close(fd_fb);
}

/*************************************************************//*
 * Backup framebuffer
 * parameter
 *         None
 *
 * return
 *         None
 ************************************************************/
static void backup_fb(void)
{
	long int inc;
	long int buf_size = vinfo.xres * vinfo.yres;

	if (!init_flag) {
		printf("LCD hasn't initialized\n");
		return;
	}

	/* Allocation memory for store old framebuffer */
	old_fbp = (short*)malloc(buf_size * (sizeof(short)));
	if (!old_fbp) {
		printf("Allocation faild\n"); 
		return;
	}

	/* Backup framebuffer */
	for(inc = 0; inc < buf_size; ++inc) {
		*(old_fbp + inc) = *(fbp + inc);
	}
}
		
/*************************************************************//*
 * Restitute framebuffer
 * parameter
 *         None
 *
 * return
 *         None
 ************************************************************/
static void restitute_fb(void)
{
	long int inc;
	long int buf_size = vinfo.xres * vinfo.yres;

	if (!old_fbp) {
		printf("Have not framebuffer, restitute faild\n");
		return;
	}

	/* Restitute framebuffer */
	for (inc = 0; inc < buf_size; ++inc) {
		*(fbp + inc) = *(old_fbp + inc);
	}
}
