#include "twi.h"
#include "i2c-dev.h"

#include <linux/types.h>

#define PAGE_SIZE 128


//extern s32 i2c_smbus_write_word_data(const struct i2c_client *client, u8 command, u16 value);
/* I2C file description */
static int fd;
/* I2C initialize flag */
static RET_STATUS init_flag = FAILD;
/* Send buffer */
static char buffer[PAGE_SIZE + 2];

/**************************************************************//*
 * Initialize i2c device
 * parameter
 *         dev_pathname: The pathname of i2c device
 *         slave_addr  : Address of slave device
 *
 * return
 *         Return the status of initialize, FAILD represent
 *         initialize hasn't complate and SUCCESS represent
 *         successful
 ***************************************************************/
RET_STATUS i2c_init(const char *dev_pathname,
					long slave_addr)
{
	if (!dev_pathname) {
		return FAILD;
	}

	/* Open i2c device */
	fd = open(dev_pathname, O_RDWR);
	if (fd < 0) {
		perror("[info] Open i2c device error");
		return FAILD;
	}

	/* Set i2c addresses with normal 7 bit */
	if (ioctl(fd, I2C_TENBIT, 0) < 0) {
		perror("[info] Ioctl i2c TENBIT error");
		return FAILD;
	}

	/* Set slave device address */
	if (ioctl(fd, I2C_SLAVE_FORCE, slave_addr) < 0) {
		perror("[info] Ioctl i2c slave error");
		return FAILD;
	}

	init_flag = SUCCESS;

	return SUCCESS;
}
	
/**************************************************************//*
 * Write data to i2c slave device
 * parameter
 *         fd      : The file description of i2c device
 *         send_buf: The data that will be writed to slave device
 *         addr    : The address that start position of writing
 *         num     : The number of bytes will be writed that in
 *                   the send_buf
 *
 * return
 *         Return the status of writing, FAILD represent writing
 *         hasn't complate and SUCCESS represent successful
 ***************************************************************/
RET_STATUS i2c_write(const char *send_buf,
					 int addr,
					 int *num)
{
	int send_bytes = 0;
	int count      = 0;
	int res;
	int i;
	char buf[3];

	if (init_flag == FAILD) {
		printf("[info] I2C hasn't initialized.\n");
		return FAILD;
	}

	if (!send_buf) {
		return FAILD;
	}

	if (*num > strlen(send_buf)) {
		*num = strlen(send_buf);
	}


	for (i = 0; i < *num; ++i,++addr) {
		buf[0] = addr >> 8;
		buf[1] = addr;
		buf[2] = *(send_buf + i);

		res = i2c_smbus_write_word_data(fd, buf[0], buf[2] << 8 | buf[1]);
		usleep(5000);

		if (res < 0) {
			perror("[info] Writing occur error");
			return FAILD;
		}
		count += res;
	}
			
	*num = count;

	return SUCCESS;
}

/**************************************************************//*
 * Read data from i2c slave device
 * parameter
 *         fd      : The file description of i2c device
 *         recv_buf: It store data that will be read from i2c
 *                   slave device
 *         addr    : The address that start position of reading
 *         num     : The number of bytes will be readed
 *
 * return
 *         Return the status of reading, FAILD represent reading
 *         hasn't complate and SUCCESS represent successful
 ***************************************************************/
RET_STATUS i2c_read(char *recv_buf,
					int addr,
					const int buf_size,					
					int *num)
{
	char buf[2];
	int res;
	int i;
	int count = 0;

	if ((recv_buf == NULL) ||
		(buf_size < 0) ||
		(*num < 0)) {
		return FAILD;
	}
	
	buf[0] = addr >> 8;
	buf[1] = addr;

	res = i2c_smbus_write_byte_data(fd, buf[0], buf[1]);
	if (res < 0) {
		perror("[info] Writing address occur error");
		return FAILD;
	}
	
	for (i = 0; i < *num; ++i) {
		*(recv_buf + i) = i2c_smbus_read_byte(fd);
		if (*(recv_buf + i) < 0) {
			perror("[info] Reading occur error");
			break;
		}

		++count; 
	}

	*(recv_buf + count) = '\0';
	
	*num = count;
	if (*num > count) {
		return FAILD;
	}

	return SUCCESS;
}

/**************************************************************//*
 * Deinitialize i2c device
 * parameter
 *         None
 *
 * return
 *         None
 ***************************************************************/
void i2c_deinit(void)
{
	close(fd);
}

