#include "rs485.h"
#include "pthread.h"

#ifndef TIOCSRS485
	#define TIOCSRS485     0x542F
#endif

#ifndef TIOCGRS485
	#define TIOCGRS485     0x542E
#endif

/* Device file description */
int fd;

/* Device old attribute */
static struct termios tty_oldattr;

/* Mutex */
pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;

/***************************************************************//*
 * Initialize rs485
 * parameter
 *		pathname: The device will be opened
 *		speed: The baudrate will be setted 
 *
 * return
 *		If initialize success, return file description of the device
 *		otherwise return a negative number
 ****************************************************************/
int rs485_init(const char *pathname, const speed_t speed)
{
	struct termios tty_attr;

	/* Open device with RW, NOCTTY, NDELAY */
	if ((fd = open(pathname, O_RDWR | O_NOCTTY)) < 0) {
		fprintf(stderr, "Open %s error on %s\n", pathname, strerror(errno));
		return fd;
	} 

	/* Get the attributes from device */
	bzero(&tty_attr, sizeof(tty_attr));
	if (tcgetattr(fd, &tty_oldattr)) {
		perror("Error on getting termios attribute:");
		close(fd);
		return -1;
	}


	/* Control mode: Configure with Enable receiver, 8 data bit, */
	/* stop bit, disable parity and speed with parameter */
	tty_attr.c_cflag &= ~(CSIZE | CSTOPB | PARENB | CBAUD);
	tty_attr.c_cflag |= (speed | CS8 | CLOCAL | CREAD);
	
	/* Input mode: Configure with Ignore framing errors and parity errors */
	tty_attr.c_iflag |= IGNPAR;

	/* Output mode: Configure with 0 */
	tty_attr.c_oflag = 0;

	/* Local mode: Configure with 0 */
	tty_attr.c_lflag = 0;
	
	/* Mininum number of characters for noncanonical read setted with '0' */
	/* Timeout in deciseconds for noncanonical setted with '0' */
	tty_attr.c_cc[VMIN]  = 1;
	tty_attr.c_cc[VTIME] = 0;

	/* Flushes buffer */
	tcflush(fd, TCIOFLUSH);

	/* Set the attributes to device */
	if (tcsetattr(fd, TCSANOW, &tty_attr)) {
		perror("Error on setting attrribute:");
		close(fd);
		return -1;
	}

	/*	if(fcntl(fd, F_SETFL, 0) < 0) {
		perror("Error on setting F_SETFL:");
		close(fd);
		return -1;
	}
	*/
	return fd;
}

/***************************************************************//*
 * Enable or Disable rs485 mode 
 * parameter
 *		enable: The flag of enable of disable rs485 mode
 *
 * return
 *		The result of setting, if setting occur error,return 
 *		a negative number
 ***************************************************************/
int rs485_enable(const RS485_ENABLE_t enable)
{
	struct serial_rs485 rs485conf;
	int res;

	/* Get configure from device */
	res = ioctl(fd, TIOCGRS485, &rs485conf);
	if (res < 0) {
		perror("Ioctl error on getting 485 configure:");
		close(fd);
		return res;
	}

	/* Set enable/disable to configure */
	if (enable) {	// Enable rs485 mode
		rs485conf.flags |= SER_RS485_ENABLED;
	} else {		// Disable rs485 mode
		rs485conf.flags &= ~(SER_RS485_ENABLED);
	}

	rs485conf.delay_rts_before_send = 0x00000004;

	/* Set configure to device */
	res = ioctl(fd, TIOCSRS485, &rs485conf);
	if (res < 0) {
		perror("Ioctl error on setting 485 configure:");
		close(fd);
	}

	return res;
}

/***************************************************************//*
 * Write data to rs485 
 * parameter
 *		tx_buf: The content of writing
 *		num: How many bytes of in tx_buf will be writed
 *
 * return
 *		The number of the write bytes, if writting occur error,
 *		return -1
 **************************************************************/
ssize_t rs485_write(const char *tx_buf, const ssize_t num)
{
	ssize_t res;
	
	/* Write data to rs485 */
	pthread_mutex_lock(&mutex);
	
	if ((res = write(fd, tx_buf, strlen(tx_buf))) == -1) {
		fprintf(stderr, "Write error on %s\n", strerror(errno));
	}

	pthread_mutex_unlock(&mutex);
	
	return res;
}

/***************************************************************//*
 * Read data from rs485
 * parameter
 *		rx_buf: Store the reading content
 *		num: How many bytes will be readed
 * return
 *		The number of reading bytes, if reading occur error,
 *		return errno
 ***************************************************************/
ssize_t rs485_read(char *rx_buf, const ssize_t num)
{
	ssize_t res = 0;

	/* Get bytes number in buffer */
	while (res < 1) {
		if (ioctl(fd, FIONREAD, &res) < 0) {
			perror("Error on get bytes number in buffer");
			return errno;
		} else {
			usleep(100000);
		}
	}

	/* Read data from rs485 */
	pthread_mutex_lock(&mutex);
	
	(res > num) ? (res = num) : res;
	if ((res = read(fd, rx_buf, res)) == -1) {
		fprintf(stderr, "Read error on %s\n", strerror(errno));
	} else {
		rx_buf[res] = '\0';
	}

	pthread_mutex_unlock(&mutex);
	
	return res;
}

/**************************************************************//*
 * Clean resource when exit rs485
 * parameter
 *      None
 *
 * return
 *      None
 *************************************************************/
void rs485_exit(void)
{
	/* Restitute device attribute */
	if (tcsetattr(fd, TCSANOW, &tty_oldattr) == -1) {
		perror("Restitute error:");
	}

	/* Close the file description of device */
	close(fd);
	pthread_mutex_destroy(&mutex);
}
